@@ -2,6 +2,8 @@ package C::sparse::expand::EXPANSION_CMDLINE;
 our @ISA = qw (C::sparse::expand);
 package C::sparse::expand::EXPANSION_STREAM;
 our @ISA = qw (C::sparse::expand);
+package C::sparse::expand::EXPANSION_MACRODEF;
+our @ISA = qw (C::sparse::expand);
 package C::sparse::expand::EXPANSION_MACRO;
 our @ISA = qw (C::sparse::expand);
 package C::sparse::expand::EXPANSION_MACROARG;
@@ -10,5 +12,7 @@ package C::sparse::expand::EXPANSION_CONCAT;
 our @ISA = qw (C::sparse::expand);
 package C::sparse::expand::EXPANSION_PREPRO;
 our @ISA = qw (C::sparse::expand);
+package C::sparse::expand::EXPANSION_SUBST;
+our @ISA = qw (C::sparse::expand);
 
 1;
