@@ -46,6 +46,7 @@ foreach my $v (qw(
 	TOKEN_IF
 	TOKEN_SKIP_GROUPS
 	TOKEN_ELSE
+	TOKEN_CONS
 
 	SPECIAL_BASE
 	SPECIAL_ADD_ASSIGN
@@ -111,10 +112,12 @@ foreach my $v (qw(
 
 	EXPANSION_CMDLINE
 	EXPANSION_STREAM
+        EXPANSION_MACRODEF
 	EXPANSION_MACRO
 	EXPANSION_MACROARG
 	EXPANSION_CONCAT
 	EXPANSION_PREPRO
+	EXPANSION_SUBST
 
         CONSTANT_FILE_MAYBE
         CONSTANT_FILE_IFNDEF
